<?php

namespace Drupal\cool_calendar_extras\Plugin\Validation\Constraint;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

/**
 * Class to validate MaxRecurrences constraint in MaxRecurrencesConstraint.php.
 */
class MaxRecurrencesConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  use StringTranslationTrait;


  /**
   * The entity type manager, used to fetch entity link templates.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The string translation service.
   *
   * @var Drupal\Core\StringTranslation\TranslationInterface
   */
  protected $stringTranslation;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, ConfigFactoryInterface $configFactory, TranslationInterface $stringTranslation) {
    $this->entityTypeManager = $entityTypeManager;
    $this->configFactory = $configFactory;
    $this->stringTranslation = $stringTranslation;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('config.factory'),
      $container->get('string_translation')
    );
  }

  /**
   * This function will execute every time that the user save a entity/node.
   */
  public function validate($value, Constraint $constraint) {

    // $value --> in this variable we have the node.
    // $constraint --> the class information MaxRecurrencesConstraint.php.
    // Check if it is a node.
    if ($value->getEntityTypeId() != 'node') {
      // Is not a node, discontinue the validation.
      return;
    }

    // Get the configuration that is set into administration section.
    $config = $this->configFactory->get('cool_calendar_extras.settings');

    // Get the module configuration.
    if (is_null($config)) {
      // No module configuration exist, discontinue the validation.
      return;
    }

    // Generate the fields names that they should have.
    $name_module = 'cool_calendar_extras';

    $config_smartdate = $name_module . '_' . $value->bundle() . '_constraint' . '_smartdate';
    $config_recurrences_count = $name_module . '_' . $value->bundle() . '_constraint' . '_recurrences_count';

    // Check if some restriction is configured for this type of content.
    if (is_null($config->get($config_smartdate)) || $config->get($config_smartdate) == 'none') {
      // No restriction present, discontinue the validation.
      return;
    }

    // Check if is set a max recurrence configuration.
    if (is_null($config->get($config_recurrences_count)) || $config->get($config_recurrences_count) == 0) {
      // It is not present, discontinue the validation.
      return;
    }

    // Get the max recurrences allowed by this type of content.
    $max_recurrences_allowed = $config->get($config_recurrences_count);

    // Get the number of recurrences from the node (that has set the user)
    $num_recurrences_user_set = $this->getNumRecurrences($value, $config->get($config_smartdate));

    // If the user has set more recurrences than recurrences allowed by
    // this type of content, it does not pass de validation.
    if ($num_recurrences_user_set > $max_recurrences_allowed) {

      $violation_str = $this->t('The recurrence can not exceeds the max number allowed:') . ' ' . $max_recurrences_allowed . ' ' . $this->t('recurrences.') . '<br>';
      $violation_str .= $this->t('Your configuration generates:') . ' ' . $num_recurrences_user_set . ' ' . $this->t('recurrences.');

      $this->context->addViolation($violation_str);

    }

  }

  /**
   * Get the number of recurrences from a SmartDate field.
   */
  private function getNumRecurrences($node, $smart_date_field_name) {

    // Get the entity type name.
    $node_entity_type = $node->getEntityTypeId();

    // Get the bundle.
    $node_bundle = $node->bundle();

    // Get the smart_date field value.
    $smart_date_field_value = $node->get($smart_date_field_name)->getValue();

    // Get the smart_date field configuration, the limit of months
    // that will be able to generate the occurrences.
    $smart_date_field_config = $this->entityTypeManager->getStorage('field_config')->load($node_entity_type . '.' . $node_bundle . '.' . $smart_date_field_name);
    // If the value doesn't exist or is NULL, by default set it to 12 months.
    $month_limit = $smart_date_field_config->getThirdPartySetting('smart_date_recur', 'month_limit', 12);

    // Use next function (from smart_date_recur.module) that will return an
    // array with all occurrences of the field that are saved.
    $recurrences = smart_date_recur_generate_rows($smart_date_field_value, $node_entity_type, $node_bundle, $smart_date_field_name, $month_limit);

    // Return the number of recurrences from the SmartDate field.
    return count($recurrences);

  }

}
